package org.xbib.interlibrary.http.server.search;

import org.xbib.interlibrary.action.search.SearchRequest;
import org.xbib.interlibrary.action.search.SearchResponse;
import org.xbib.interlibrary.api.action.Response;
import org.xbib.interlibrary.http.server.BaseView;
import org.xbib.j2html.tags.DomContent;
import org.xbib.j2html.tags.DomContentJoiner;
import org.xbib.j2html.tags.Tag;
import org.xbib.marc.MarcField;
import org.xbib.marc.MarcRecord;
import org.xbib.net.ParameterException;
import org.xbib.net.http.server.HttpRequest;
import org.xbib.net.http.server.application.Application;
import org.xbib.net.http.server.resource.HtmlTemplateResource;
import org.xbib.net.http.server.route.HttpRouterContext;
import org.xbib.net.http.server.session.Session;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import static org.xbib.j2html.TagCreator.div;
import static org.xbib.j2html.TagCreator.each;
import static org.xbib.j2html.TagCreator.main;
import static org.xbib.j2html.TagCreator.span;
import static org.xbib.j2html.TagCreator.strong;
import static org.xbib.j2html.TagCreator.table;
import static org.xbib.j2html.TagCreator.tbody;
import static org.xbib.j2html.TagCreator.td;
import static org.xbib.j2html.TagCreator.th;
import static org.xbib.j2html.TagCreator.thead;
import static org.xbib.j2html.TagCreator.tr;

public class RecordView extends BaseView {

    public RecordView(HtmlTemplateResource resource,
                      Application application,
                      HttpRequest request) throws ParameterException {
        super(resource, application, request);
    }

    @Override
    protected DomContent mainContent(HttpRouterContext context, Tag<?>... tags) {
        SearchWebApplicationModule app = context.getAttributes().get(SearchWebApplicationModule.class, "app");
        Objects.requireNonNull(app, "app must not be null in context");
        Session session = context.getAttributes().get(Session.class, "session");
        Objects.requireNonNull(session, "session must not be null in context");
        return main(recordView(app.getSearchRequest(request)));
    }

    private DomContent recordView(SearchResponse<SearchRequest> searchResponse) {
        if (searchResponse == null) {
            return div().withText("Es konnte kein Datensatz gefunden werden.");
        }
        if (searchResponse.getStatus() != Response.Status.OK) {
            return div().withText("Es trat ein Fehler auf.");
        }
        return div().attr("class", "container").with(
                each(searchResponse.getData().getMap(), e ->
                    DomContentJoiner.join("", false,
                            leader(e), fields(e))
                )
        );
    }

    private DomContent leader(Map.Entry<String, List<Map<String, Object>>> entry) {
        Map<String, Object> map = entry.getValue().getFirst();
        String leader = (String) map.get("_leader");
        return table().attr("class", "table table-sm").with(
                thead().with(th().withText("Satzkennung")),
                tbody().with(tr().with(td().with(strong().withText(leader))))
        );
    }

    private DomContent fields(Map.Entry<String, List<Map<String, Object>>> entry) {
        Map<String, Object> map = entry.getValue().getFirst();
        MarcRecord marcRecord = (MarcRecord) map.get("_source");
        return table().attr("class", "table table-sm").with(
                thead().with(
                        th().withText("Feld"),
                        th().withText("Indikator"),
                        th().withText("Inhalt")
                ),
                tbody().with(
                        each(marcRecord.getFields(), f ->
                                tr().with(
                                        td().with(strong().withText(f.getTag())),
                                        td().with(strong().withText(indicator(f))),
                                        td().with(content(f))
                                ))
                )
        );
    }

    private String indicator(MarcField marcField) {
        return marcField.getIndicator() != null ? "#" + marcField.getIndicator().replace(' ', '_') : "";
    }

    private DomContent content(MarcField marcField) {
        if (marcField.getValue() != null) {
            return span().withText(marcField.getValue());
        }
        List<DomContent> list = new ArrayList<>();
        for (MarcField.Subfield sf : marcField.getSubfields()) {
            list.add(strong().withText("$" + sf.getId() + " "));
            list.add(span().withText(sf.getValue() + " "));
        }
        return DomContentJoiner.join("", false, list.toArray());
    }
}
