package org.xbib.interlibrary.http.server.search;

import org.xbib.interlibrary.http.server.BaseView;
import org.xbib.j2html.tags.DomContent;
import org.xbib.j2html.tags.Tag;
import org.xbib.net.ParameterException;
import org.xbib.net.http.server.HttpRequest;
import org.xbib.net.http.server.application.Application;
import org.xbib.net.http.server.resource.HtmlTemplateResource;
import org.xbib.net.http.server.route.HttpRouterContext;
import org.xbib.net.http.server.session.Session;

import java.util.List;
import java.util.Map;
import java.util.Objects;

import static org.xbib.j2html.TagCreator.div;
import static org.xbib.j2html.TagCreator.each;
import static org.xbib.j2html.TagCreator.form;
import static org.xbib.j2html.TagCreator.input;
import static org.xbib.j2html.TagCreator.main;
import static org.xbib.j2html.TagCreator.option;
import static org.xbib.j2html.TagCreator.select;
import static org.xbib.j2html.TagCreator.span;

public class SearchFormView extends BaseView {

    public SearchFormView(HtmlTemplateResource resource,
                          Application application,
                          HttpRequest request) throws ParameterException {
        super(resource, application, request);
    }

    @SuppressWarnings("unchecked")
    @Override
    protected DomContent mainContent(HttpRouterContext context,
                                     Tag<?>... tags) {
        SearchWebApplicationModule app = context.getAttributes().get(SearchWebApplicationModule.class, "app");
        Objects.requireNonNull(app, "app must not be null in context");
        Session session = context.getAttributes().get(Session.class, "session");
        Objects.requireNonNull(session, "session must not be null in context");
        Map<String, Object> searchform = app.getForm("searchform", request, session);
        String formId = (String) searchform.get("_id");
        Map<String, Object> searchresponse = session.isValid() ?
                (Map<String, Object>) session.get("_searchresponse") : Map.of();
        SearchResultView searchResultView = new SearchResultView(resource, request, formId, app.getSearchServices());
        return main(searchForm(searchform), searchResultView.searchResult(searchform, searchresponse));
    }

    @SuppressWarnings("unchecked")
    private DomContent searchForm(Map<String, Object> searchform) {
        List<SearchForm.Key> keys1 = (List<SearchForm.Key>) searchform.get("keys1");
        List<SearchForm.Key> keys2 = (List<SearchForm.Key>) searchform.get("keys2");
        List<SearchForm.Key> keys3 = (List<SearchForm.Key>) searchform.get("keys3");
        List<SearchForm.Key> ops2 = (List<SearchForm.Key>) searchform.get("ops2");
        List<SearchForm.Key> ops3 = (List<SearchForm.Key>) searchform.get("ops3");
        List<SearchForm.Key> databases = (List<SearchForm.Key>) searchform.get("databases");
        List<SearchForm.Key> sorts = (List<SearchForm.Key>) searchform.get("sorts");
        List<SearchForm.Key> hints = (List<SearchForm.Key>) searchform.get("hints");
        return div().attr("class", "container").with(
                form().attr("class", "form-horizontal mt-2 mb-2")
                        .attr("id", "searchform")
                        .attr("method", "post")
                        .attr("action", resource.contextPath(request, "action/form")).with(
                                div().attr("class", "row mb-2 input-group").with(
                                        span().attr("class", "col-sm-3").with(
                                                select().attr("class", "form-select")
                                                .attr("id", "key1")
                                                .attr("name", "key1")
                                                .with(each(keys1, entry ->
                                                        option().attr("value", entry.getValue())
                                                                .attr(entry.isDisabled() ? "disabled" : entry.isSelected() ? "selected" : "")
                                                                .withText(entry.getLabel())
                                                ))
                                        ),
                                        span().attr("class", "col-sm-7").with(
                                                input().attr("class", "form-control")
                                                        .attr("type", "text")
                                                        .attr("name", "value1")
                                                        .attr("value", searchform.get("value1"))
                                        ),
                                        span().attr("class", "col-sm-2").with(
                                                select().attr("class", "form-select")
                                                .attr("id", "op2")
                                                .attr("name", "op2")
                                                .with(each(ops2, entry ->
                                                                option().attr("value", entry.getValue())
                                                                        .attr(entry.isDisabled() ? "disabled" : entry.isSelected() ? "selected" : "")
                                                                        .withText(entry.getLabel())
                                                ))
                                        )
                                ),
                                div().attr("class", "row mb-2 input-group").with(
                                        span().attr("class", "col-sm-3").with(
                                                select().attr("class", "form-select")
                                                        .attr("id", "key2")
                                                        .attr("name", "key2")
                                                        .with(each(keys2, entry ->
                                                                option().attr("value", entry.getValue())
                                                                        .attr(entry.isDisabled() ? "disabled" : entry.isSelected() ? "selected" : "")
                                                                        .withText(entry.getLabel())
                                                        ))
                                        ),
                                        span().attr("class", "col-sm-7").with(
                                                input().attr("class", "form-control")
                                                        .attr("type", "text")
                                                        .attr("name", "value2")
                                                        .attr("value", searchform.get("value2"))

                                        ),
                                        span().attr("class", "col-sm-2").with(
                                                select().attr("class", "form-select")
                                                        .attr("id", "op3")
                                                        .attr("name", "op3")
                                                        .with(each(ops3, entry ->
                                                                option().attr("value", entry.getValue())
                                                                        .attr(entry.isDisabled() ? "disabled" : entry.isSelected() ? "selected" : "")
                                                                        .withText(entry.getLabel())
                                                        ))
                                        )
                                ),
                                div().attr("class", "row mb-2 input-group").with(
                                        span().attr("class", "col-sm-3").with(
                                                select().attr("class", "form-select")
                                                        .attr("id", "key3")
                                                        .attr("name", "key3")
                                                        .with(each(keys3, entry ->
                                                                option().attr("value", entry.getValue())
                                                                        .attr(entry.isDisabled() ? "disabled" : entry.isSelected() ? "selected" : "")
                                                                        .withText(entry.getLabel())
                                                        ))
                                        ),
                                        span().attr("class", "col-sm-7").with(
                                                input().attr("class", "form-control")
                                                        .attr("type", "text")
                                                        .attr("name", "value3")
                                                        .attr("value", searchform.get("value3"))

                                        )
                                ),
                                div().attr("class", "row mb-2 input-group").with(
                                        span().attr("class", "col-sm-2 form-label")
                                                .withText("Datenbankauswahl"),
                                        span().attr("class", "col-sm-10")
                                                .with(each(databases, d ->
                                                        span().attr("class", "form-check form-check-inline").with(
                                                                input().attr("class", "form-check-input")
                                                                        .attr("type", "checkbox")
                                                                        .attr("name", "db")
                                                                        .attr("value", d.getValue())
                                                                        .attr(d.isDisabled() ? "disabled" : d.isSelected() ? "checked" : ""),
                                                                span().attr("class", "form-check-label").withText(d.getLabel())
                                                        )),
                                                        span().attr("class", "ps-1"),
                                                        input().attr("class", "btn btn-outline-secondary btn-sm")
                                                                .attr("type", "button")
                                                                .attr("value", "keine")
                                                                .attr("onclick", "javascript:clearCheckBoxes('searchform', 'db')"),
                                                        span().attr("class", "ps-1"),
                                                        input().attr("class", "btn btn-outline-secondary btn-sm")
                                                                .attr("type", "button")
                                                                .attr("value", "alle")
                                                                .attr("onclick", "javascript:setCheckBoxes('searchform', 'db')")
                                                )
                                ),
                                div().attr("class", "row mb-2").with(
                                        span().attr("class", "col-sm-2 form-label")
                                                .withText("Trefferanzahl/Datenbank"),
                                        span().attr("class", "col-sm-1").with(
                                            input().attr("class", "form-control")
                                                .attr("name", "size")
                                                .attr("type", "number")
                                                .attr("min", "1")
                                                .attr("max", "100")
                                                .attr("value", searchform.get("size"))
                                        ),
                                        span().attr("class", "col-sm-1 form-label")
                                                .withText("Sortierung"),
                                        span().attr("class", "col-sm-2").with(
                                                        select().attr("class", "form-select")
                                                                .attr("id", "sort")
                                                                .attr("name", "sort").with(
                                                                        each(sorts, sort ->
                                                                                option().attr("value", sort.getValue())
                                                                                .attr(sort.isDisabled() ? "disabled" : sort.isSelected() ? "selected" : "")
                                                                                .withText(sort.getLabel())
                                                                ))
                                        ),
                                        span().attr("class", "col-sm-1 form-label")
                                                .withText("Hinweis"),
                                        span().attr("class", "col-sm-3")
                                                .with(each(hints, d ->
                                                                span().attr("class", "form-check form-check-inline").with(
                                                                        input().attr("class", "form-check-input")
                                                                                .attr("type", "checkbox")
                                                                                .attr("name", "hint")
                                                                                .attr("value", d.getValue())
                                                                                .attr(d.isDisabled() ? "disabled" : d.isSelected() ? "checked" : ""),
                                                                        span().attr("class", "form-check-label").withText(d.getLabel())
                                                                )),
                                                        span().attr("class", "ps-1"),
                                                        input().attr("class", "btn btn-outline-secondary btn-sm")
                                                                .attr("type", "button")
                                                                .attr("value", "keine")
                                                                .attr("onclick", "javascript:clearCheckBoxes('searchform', 'hint')"),
                                                        span().attr("class", "ps-1"),
                                                        input().attr("class", "btn btn-outline-secondary btn-sm")
                                                                .attr("type", "button")
                                                                .attr("value", "alle")
                                                                .attr("onclick", "javascript:setCheckBoxes('searchform', 'hint')")
                                                )
                                        ),
                                div().attr("class", "d-grid gap-2 d-md-flex justify-content-md-end").with(
                                        input().attr("type", "hidden")
                                                .attr("name", "searchform")
                                                .attr("value", searchform.get("_id")),
                                        input().attr("type", "hidden")
                                                .attr("name", "offset")
                                                .attr("value", "1"),
                                        input().attr("type", "hidden")
                                                .attr("name", "page")
                                                .attr("value", "1"),
                                        input().attr("class", "btn btn-outline-primary")
                                                .attr("type", "submit")
                                                .attr("id", "searchbutton")
                                                .attr("name", "searchbutton")
                                                .attr("value", "Suchen >>")
                                                .attr("onclick", "javascript:pleaseWait('searchbutton', 'searchform')")
                                )
                        )
                );
    }

}
