package org.xbib.interlibrary.http.server.search;

import java.io.IOException;
import java.util.List;
import org.xbib.datastructures.common.MultiMap;
import org.xbib.interlibrary.action.search.DefaultSearchRequester;
import org.xbib.interlibrary.action.search.SearchAction;
import org.xbib.interlibrary.action.search.SearchRequest;
import org.xbib.interlibrary.action.search.SearchRequester;
import org.xbib.interlibrary.action.search.SearchResponse;
import org.xbib.interlibrary.action.search.SearchSyntax;
import org.xbib.interlibrary.api.ApplicationModule;
import org.xbib.interlibrary.common.Interlibrary;

import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.xbib.interlibrary.common.InterlibraryModule;

public class SearchModule implements InterlibraryModule {

    private static final Logger logger = Logger.getLogger(SearchModule.class.getName());

    private Interlibrary interlibrary;

    public SearchModule() {
    }

    @Override
    public String getName() {
        return "SearchModule";
    }

    @Override
    public List<Class<? extends ApplicationModule>> dependsOn() {
        return null;
    }

    @Override
    public void setInterlibrary(Interlibrary interlibrary) {
        this.interlibrary = interlibrary;
    }

    @Override
    public Interlibrary getInterlibrary() {
        return interlibrary;
    }

    @SuppressWarnings("unchecked")
    public SearchResponse<SearchRequest> search(MultiMap<String, Object> params) {
        SearchRequest.Builder builder = SearchRequest.builder();
        if (params.containsKey(SearchParameters.SERVICE)) {
            builder.setServices(params.get(SearchParameters.SERVICE));
        }
        if (params.containsKey(SearchParameters.ENDPOINT)) {
            builder.setEndpoint(params.getString(SearchParameters.ENDPOINT));
        }
        if (params.containsKey("_content")) {
            builder.setContent((String) params.get("_content").iterator().next());
        }
        if (params.containsKey("_map")) {
            builder.setMap((Map<String, Object>) params.get("_map").iterator().next());
        }
        SearchRequester requester = new DefaultSearchRequester(params.getString(SearchParameters.REQUESTER, ""));
        builder.setRequester(requester);
        builder.setSearchSyntax(SearchSyntax.NATIVE);
        if (params.containsKey(SearchParameters.SYNTAX)) {
            try {
                // we can choose search syntax
                builder.setSearchSyntax(SearchSyntax.valueOf(params.getString(SearchParameters.SYNTAX)));
            } catch (IllegalArgumentException e) {
                // fallback is always NATIVE
                builder.setSearchSyntax(SearchSyntax.NATIVE);
            }
        }
        builder.setSearchOp(params.get(SearchParameters.OP));
        builder.setSearchKey(params.get(SearchParameters.KEY));
        builder.setSearchValue(params.get(SearchParameters.QUERY));
        builder.setOffset(params.getInteger(SearchParameters.OFFSET, 0));
        builder.setSize(params.getInteger(SearchParameters.SIZE, 10));
        if (params.containsKey("hint")) {
            for (Object hint : params.get("hint")) {
                builder.addHint(hint);
            }
        }
        if (params.containsKey("extended")) {
            builder.setExtended(params.getBoolean("extended", false));
        }
        builder.setSourceEnabled(true);
        if (params.containsKey("debug")) {
            builder.setDebug(params.getBoolean("debug", false));
        }
        builder.setValidated(false);
        SearchRequest searchRequest = builder.build();
        logger.log(Level.INFO, "request = " + searchRequest.toString());
        return search(searchRequest);
    }

    public SearchResponse<SearchRequest> search(SearchRequest searchRequest) {
        return interlibrary.executeReduce(SearchAction.INSTANCE, searchRequest);
    }

    @Override
    public void run() {

    }

    @Override
    public void close() throws IOException {
    }
}
