package org.xbib.interlibrary.http.server.search;

import org.xbib.interlibrary.action.search.SearchService;
import org.xbib.interlibrary.api.library.Material;
import org.xbib.j2html.tags.ContainerTag;
import org.xbib.j2html.tags.DomContent;
import org.xbib.j2html.tags.DomContentJoiner;
import org.xbib.j2html.tags.specialized.SpanTag;
import org.xbib.net.URL;
import org.xbib.net.http.server.HttpRequest;
import org.xbib.net.http.server.resource.HtmlTemplateResource;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.LongStream;

import static org.xbib.j2html.TagCreator.a;
import static org.xbib.j2html.TagCreator.div;
import static org.xbib.j2html.TagCreator.each;
import static org.xbib.j2html.TagCreator.i;
import static org.xbib.j2html.TagCreator.img;
import static org.xbib.j2html.TagCreator.li;
import static org.xbib.j2html.TagCreator.nav;
import static org.xbib.j2html.TagCreator.p;
import static org.xbib.j2html.TagCreator.rawHtml;
import static org.xbib.j2html.TagCreator.span;
import static org.xbib.j2html.TagCreator.strong;
import static org.xbib.j2html.TagCreator.ul;

public class SearchResultView {

    private final HtmlTemplateResource resource;

    private final HttpRequest request;

    private final String formId;

    private final List<SearchService> searchServices;

    public SearchResultView(HtmlTemplateResource resource,
                            HttpRequest request,
                            String formId,
                            List<SearchService> searchServices) {
        this.resource = resource;
        this.request = request;
        this.formId = formId;
        this.searchServices = searchServices;
    }

    @SuppressWarnings("unchecked")
    public DomContent searchResult(Map<String, Object> searchForm,
                                   Map<String, Object> searchResult) {
        if (searchResult == null) {
            return emptyResult();
        }
        // paging control
        int limit = 1000;
        int offset = Integer.parseInt((String) searchForm.get("offset"));
        if (offset == 0) {
            offset = 1;
        }
        if (offset > limit) {
            offset = limit;
        }
        int size = Integer.parseInt((String) searchForm.get("size"));
        if (size == 0) {
            size = 1;
        }
        if (size > limit) {
            size = limit;
        }
        long page = searchForm.containsKey("page") ? Long.parseLong((String) searchForm.get("page")) : 0L;
        long currentpage = page != 0L ? page : (offset / size);
        if (currentpage == 0L) {
            currentpage = 1L;
        }
        if (currentpage > limit) {
            currentpage = limit;
        }
        long total = searchResult.containsKey("total") ? (Long) searchResult.get("total") : 0L;
        int pagelimit = size;
        int pages = 5;
        long lastpage = total != 0 ? total / pagelimit : 0L;
        if (total != 0L && (total % pagelimit != 0L)) {
            lastpage++;
        }
        long first = Math.max(1, currentpage - 2);
        long last = Math.min(first + pages, lastpage);
        Map<String, Object> metadata = (Map<String, Object>) searchResult.get("metadata");
        List<Map<String, Object>> data = (List<Map<String, Object>>) searchResult.get("data");
        if (metadata == null || data == null || metadata.isEmpty() || data.isEmpty()) {
            return emptyResult();
        }
        return div().attr("class", "container").with(
                meta(metadata, total, currentpage, lastpage),
                hits(data),
                pagination(page, first, last, lastpage)
        );
    }

    private static DomContent emptyResult() {
        return div().attr("class", "container").with(
                div().attr("class", "row").with(
                        div().attr("class", "col-md-12").with(
                                p("Es konnten keine Ergebnisse gefunden werden.")
                        )
                )
        );
    }

    private DomContent meta(Map<String, Object> metadata, long total, long currentpage, long lastpage) {
        return div().attr("class", "row").with(
                div().attr("class", "col-md-12").with(
                        span().withText("Treffer: " + total),
                        span().withText(total > 0 ? " Seite " + currentpage + " von " + lastpage : ""),
                        div().attr("class", "small").with(
                                each(metadata, meta ->
                                        DomContentJoiner.join("", false,
                                                span().withText(meta.getKey().toUpperCase(Locale.ROOT)),
                                                span().withText(": "),
                                                span().withText(getState(meta)),
                                                span().withText(" ("),
                                                span().withText(getTotal(meta)),
                                                span().withText(" Treffer) "),
                                                span().withText(getMessage(meta))
                                        )
                                )
                        )
                )
        );
    }

    private DomContent hits(List<Map<String, Object>> data) {
        return div().attr("class", "row").with(
                div().attr("class", "col-md-12").with(
                        each(data, d ->
                                div().attr("class", "mt-2").with(
                                        getHeadTitle(d),
                                        getTailTitle(d),
                                        span(" "),
                                        getMaterial(d),
                                        getCreators(d),
                                        getPublicationStatement(d),
                                        getChronology(d),
                                        getTypeAndFormatStatement(d),
                                        getPublicationIdentifiers(d),
                                        getLinks(d),
                                        getAccessLinks(d),
                                        detailButton(d)
                                )
                        )
                )
        );
    }

    private DomContent pagination(long page,
                                  long first,
                                  long last,
                                  long lastpage) {
        if (first >= last) {
            return span();
        }
        long prev = Math.max(1L, page - 1L);
        long next = Math.min(page + 1L, last);
        List<Long> pages = LongStream.range(first, last + 1).boxed().toList();
        return div().attr("class", "row").with(
                div().attr("class", "col-md-12").with(
                        nav().attr("aria-label", "Navigation").with(
                                ul().attr("class", "pagination justify-content-center").with(
                                        li().attr("class", page == 1L ? "page-item disabled" : "page-item").with(
                                                a().attr("class", "page-link")
                                                        .attr("href", resource.contextPath(request, "action/form/" + formId + "/page/1")).with(
                                                                i().attr("class", "bi-caret-left-square")
                                                        )
                                        ),
                                        li().attr("class", page == prev ? "page-item disabled" : "page-item").with(
                                                a().attr("class", "page-link")
                                                        .attr("href", resource.contextPath(request, "action/form/" + formId + "/page/" + prev)).with(
                                                                i().attr("class", "bi-caret-left")
                                                        )
                                        ),
                                        each(pages, p ->
                                                li().attr("class", page == p ? "page-item disabled" : "page-item").with(
                                                        a().attr("class", "page-link")
                                                                .attr("href", resource.contextPath(request, "action/form/" + formId + "/page/" + p)).withText(String.valueOf(p))
                                                )
                                        ),
                                        li().attr("class", page == next ? "page-item disabled" : "page-item").with(
                                                a().attr("class", "page-link")
                                                        .attr("href", resource.contextPath(request, "action/form/" + formId + "/page/" + next)).with(
                                                                i().attr("class", "bi-caret-right")
                                                        )
                                        ),
                                        li().attr("class", page == lastpage ? "page-item disabled" : "page-item").with(
                                                a().attr("class", "page-link")
                                                        .attr("href", resource.contextPath(request, "action/form/" + formId + "/page/" + lastpage)).with(
                                                                i().attr("class", "bi-caret-right-square")
                                                        )
                                        )
                                )
                        )
                )
        );
    }

    @SuppressWarnings("unchecked")
    private static String getState(Map.Entry<String, Object> entry) {
        Map<String, Object> value = (Map<String, Object>)entry.getValue();
        return STATES.get((String) value.get("status"));
    }

    @SuppressWarnings("unchecked")
    private static String getTotal(Map.Entry<String, Object> entry) {
        Map<String, Object> value = (Map<String, Object>) entry.getValue();
        return value.getOrDefault("total", "").toString();
    }

    @SuppressWarnings("unchecked")
    private static String getMessage(Map.Entry<String, Object> entry) {
        Map<String, Object> value = (Map<String, Object>) entry.getValue();
        return value.getOrDefault("message", "").toString();
    }

    private static DomContent getMaterial(Map<String, Object> value) {
        SpanTag span = span().attr("class", "me-1");
        if (value.containsKey("material")) {
            Object object = value.get("material");
            if (object instanceof Collection<?> collection) {
                List<Material> materials = collection.isEmpty() ? List.of(Material.UNSPECIFIED)
                        : collection.stream().map(Object::toString).map(Material::fromValue).toList();
                if (materials.contains(Material.TEXT)) {
                    return span.with(i().attr("class", "bi-book"));
                }
                if (materials.contains(Material.ONLINE_RESOURCE)) {
                    return span().with(i().attr("class", "bi-display"));
                }
                if (materials.contains(Material.ELECTRONIC_RESOURCE)) {
                    return span().with(i().attr("class", "bi-display"));
                }
            }
        }
        // Serie, Reihe
        if (value.containsKey("multipart")) {
            return span().with(i().attr("class", "bi-three-dots"));
        }
        return span.with(i());
    }

    @SuppressWarnings("unchecked")
    private static DomContent getHeadTitle(Map<String, Object> value) {
        List<Object> titles = (List<Object>) value.get("title");
        if (titles == null || titles.isEmpty()) {
            return span();
        }
        return strong().with(rawHtml(titles.getFirst().toString()));
    }

    @SuppressWarnings("unchecked")
    private static DomContent getTailTitle(Map<String, Object> value) {
        List<Object> titles = (List<Object>) value.get("title");
        if (titles == null || titles.isEmpty() || titles.size() == 1) {
            return span();
        }
        return span().withText(" " + String.join(", ", titles.stream().map(Object::toString).skip(1).toList()));
    }

    @SuppressWarnings("unchecked")
    private static DomContent getCreators(Map<String, Object> value) {
        List<Object> creators = (List<Object>) value.get("creator");
        if (creators == null || creators.isEmpty()) {
            return span();
        }
        return div().withText(String.join(", ", creators.stream().map(Object::toString).toList()));
    }

    private static DomContent getPublicationStatement(Map<String, Object> value) {
        List<DomContent> list = new ArrayList<>();
        for (Object object : getList(value, "in")) {
            if (object instanceof String) {
                if (!list.isEmpty()) {
                    list.add(span(", "));
                }
                list.add(span(object.toString()));
            }
        }
        for (String publishedBy : getList(value, "publishedBy")) {
            if (!publishedBy.isEmpty()) {
                if (!list.isEmpty()) {
                    list.add(span(", "));
                }
                list.add(span(publishedBy));
            }
        }
        for (String publishedAt : getList(value, "publishedAt")) {
            if (!publishedAt.isEmpty()) {
                if (!list.isEmpty()) {
                    list.add(span(", "));
                }
                list.add(span(publishedAt));
            }
        }
        List<String> dates = getList(value, "date");
        for (String date : dates) {
            if (!date.isEmpty()) {
                if (!list.isEmpty()) {
                    list.add(span(", "));
                }
                list.add(span(date));
            }
        }
        String country = (String) value.getOrDefault("country", "");
        if (!country.isEmpty()) {
            if (!list.isEmpty()) {
                list.add(span(" "));
            }
            list.add(span().withClass("badge text-bg-secondary")
                    .withText(country));
        }
        String language = (String) value.getOrDefault("language", "");
        if (!language.isEmpty()) {
            if (!list.isEmpty()) {
                list.add(span(" "));
            }
            list.add(span().withClass("badge text-bg-secondary")
                    .withText(language));
        }
        String description = getDescription(value);
        if (!description.isEmpty()) {
            if (!list.isEmpty()) {
                list.add(span(" "));
            }
            list.add(span(description));
        }
        return div().with(list);
    }

    private static DomContent getChronology(Map<String, Object> value) {
        List<String> statement = new ArrayList<>();
        List<String> publication = getList(value,"publication");
        if (!publication.isEmpty()) {
            statement.add(String.join(", ", publication));
        }
        String chronology = (String) value.getOrDefault("chronology", "");
        if (!chronology.isEmpty()) {
            statement.add(chronology);
        }
        return div().withText(String.join(" ", statement));
    }

    private ContainerTag<?> getPublicationIdentifiers(Map<String, Object> value) {
        List<DomContent> list = new ArrayList<>();
        // the ISBN
        String isbn = getISBN(value);
        if (!isbn.isEmpty()) {
            list.add(span(isbn));
        }
        List<String> issns = getISSN(value);
        List<String> zdbs = getZDB(value);
        List<String> zdbMaterials = getZDBMaterial(value);
        for (int i = 0; i < issns.size(); i++) {
            String issn = issns.get(i);
            String zdb = i < zdbs.size() ? zdbs.get(i) : null;
            String zdbMaterial = i < zdbMaterials.size() ? zdbMaterials.get(i) : null;
            if (!list.isEmpty()) {
                list.add(span(" "));
            }
            getSearchService(value).ifPresentOrElse(searchService -> {
                String href = resource.contextPath(request, "action/form/" + formId + "/issn/" + issn);
                String cl = "online_resource".equalsIgnoreCase(zdbMaterial) ? "badge text-bg-primary" : "badge text-bg-secondary";
                list.add(span().withClass(cl)
                        .with(a().withClass("link-light text-decoration-none").withHref(href)
                                .withText("ISSN " + issn)));
            }, () -> list.add(span("ISSN " + issn)));
            if (zdb != null) {
                if (!list.isEmpty()) {
                    list.add(span(" "));
                }
                String prefix = zdb.startsWith("(DE-655)") ? "HBZ" : "ZDB"; // hack, "HBZ" should never be used
                String href = "HBZ".equals(prefix) ?
                        getSearchServiceSettings(value, "hbz_url") :
                        getSearchServiceSettings(value, "zdb_url");
                if ("notfound".equals(href)) {
                    // builtin default URL
                    href = resource.contextPath(request, "view/" + prefix + "/" + zdb);
                } else {
                    href = String.format(href, zdb);
                    URL url = URL.from(href);
                    if (!url.isAbsolute()) {
                        href = resource.contextPath(request, href);
                    }
                }
                String cl = "online_resource".equalsIgnoreCase(zdbMaterial) ? "badge text-bg-primary" : "badge text-bg-secondary";
                list.add(span().withClass(cl)
                        .with(a().withClass("link-light text-decoration-none")
                                .withHref(href).withText(prefix + " " + zdb + " (" + MATERIALS.get(zdbMaterial) + ")")));
            }
        }
        String doi = getDOI(value);
        if (!doi.isEmpty()) {
            if (!list.isEmpty()) {
                list.add(span(" "));
            }
            list.add(span().withClass("badge text-bg-primary")
                    .with(a().withClass("link-light text-decoration-none").withHref(doi).withText("DOI")));
        }
        String pmid = getPMID(value);
        if (!pmid.isEmpty()) {
            if (!list.isEmpty()) {
                list.add(span(" "));
            }
            list.add(span().withClass("badge text-bg-primary")
                    .with(a().withClass("link-light text-decoration-none").withHref(pmid).withText("PMID")));
        }
        Boolean oa = getBoolean(value, "oa");
        if (oa != null && oa) {
            String oastatus = getString(value, "oa_status");
            String oaurl = getString(value, "oa_url");
            if (!list.isEmpty()) {
                list.add(span(" "));
            }
            list.add(span().withClass("badge text-bg-warning")
                    .with(a().withClass("link-light text-decoration-none").withHref(oaurl).withText("OA " + oastatus)));
        }
        Map<String, String> identifiers = getMap(value, "identifiers");
        if (identifiers != null && !identifiers.isEmpty()) {
            for (Map.Entry<String, String> me : identifiers.entrySet()) {
                if (!list.isEmpty()) {
                    list.add(span(" "));
                }
                list.add(span().withClass("badge text-bg-light border border-secondary text-secondary")
                        .withText(me.getKey().isEmpty() ? me.getValue() : me.getKey() + " " + me.getValue()));
            }
        }
        return div().with(list);
    }

    private static String getISBN(Map<String, Object> value) {
        List<String> isbn = getList(value, "identifier", "isbn");
        return isbn.isEmpty() ? "" : "ISBN " + String.join(", ", isbn);
    }

    private static List<String> getISSN(Map<String, Object> value) {
        return getList(value, "identifier", "issn");
    }

    private static List<String> getZDB(Map<String, Object> value) {
        return getList(value, "identifier", "zdb");
    }

    private static List<String> getZDBMaterial(Map<String, Object> value) {
        return getList(value, "identifier", "material");
    }

    private static String getDOI(Map<String, Object> value) {
        return String.join(", ", getList(value, "identifier", "doi"));
    }

    private static String getPMID(Map<String, Object> value) {
        return String.join(", ", getList(value, "identifier", "pmid"));
    }

    private static String getDescription(Map<String, Object> value) {
        return value.containsKey("description") ? String.join(", ", getList(value, "description")) : "";
    }

    private static DomContent getTypeAndFormatStatement(Map<String, Object> value) {
        List<DomContent> list = new ArrayList<>();
        String type = String.join(", ", getList(value, "type"));
        if (!type.isEmpty()) {
            list.add(span().withClass("badge text-bg-light border border-secondary text-secondary")
                    .withText(type));
        }
        String format = String.join(", ", getList(value, "format"));
        if (!format.isEmpty()) {
            if (!list.isEmpty()) {
                list.add(span(" "));
            }
            list.add(span().withClass("badge text-bg-light border border-secondary text-secondary")
                    .withText(format));
        }
        return div().with(list);
    }

    @SuppressWarnings("unchecked")
    private static DomContent getLinks(Map<String, Object> value) {
        List<Map<String, Object>> links = (List<Map<String, Object>>) value.get("link");
        if (links == null) {
            return span();
        }
        List<DomContent> list = new ArrayList<>();
        for (Map<String, Object> link : links) {
            String uri = (String) link.get("uri");
            String label = link.containsKey("publicnote") ? (String) link.get("publicnote") :
                    link.containsKey("nonpublicnote") ? (String) link.get("nonpublicnote") : uri;
            list.add(div().with(a().attr("href", uri).attr("target", "_blank").withText(label)));
        }
        return each(list.stream());
    }

    @SuppressWarnings("unchecked")
    private DomContent getAccessLinks(Map<String, Object> value) {
        List<Map<String, Object>> links = (List<Map<String, Object>>) value.get("accesslink");
        if (links == null) {
            return span();
        }
        List<DomContent> list = new ArrayList<>();
        for (Map<String, Object> link : links) {
            String id = (String) link.get("id");
            String label = link.containsKey("label") ? (String) link.get("label") : id;
            list.add(div().with(a().attr("href", id).withText(label)));
            String description = (String) link.get("description");
            if ("kostenfrei".equals(description)) {
                list.add(span().withText("Diese Ressource ist grün (möglicherweise kostenlos in Bibliotheken vorhanden)"));
                list.add(img().attr("src", resource.contextPath(request, "dist/img/ezb-green.png")));
            } else if (description != null && !description.isEmpty()) {
                list.add(span().withText(description));
            }
        }
        return each(list.stream());
    }

    private DomContent detailButton(Map<String, Object> value) {
        String id = (String) value.get("id");
        String servicename = (String) value.get("servicename");
        Optional<SearchService> optional = searchServices.stream().filter(s -> s.getName().equals(servicename)).findFirst();
        String href = optional.isPresent() ?
                optional.get().getURL(id) : resource.contextPath(request, "view/" + servicename + "/" + id);
        return div().attr("class", "d-grid gap-2 d-md-flex justify-content-md-end").with(
                span().withText(id),
                a().attr("class", "btn btn-outline-secondary btn-sm'")
                        .attr("role", "button")
                        .attr("href", href)
                        .withText(">>")
        );
    }

    private Optional<SearchService> getSearchService(Map<String, Object> map) {
        String servicename = (String) map.get("servicename");
        Objects.requireNonNull(servicename);
        return searchServices.stream()
                .filter(s -> s.getName().equals(servicename))
                .findFirst();
    }

    private String getSearchServiceSettings(Map<String, Object> map, String settingsKey) {
        return getSearchService(map)
                .map(searchService -> searchService.getSettings().get(settingsKey))
                .orElse("not found");
    }

    @SuppressWarnings("unchecked")
    private static List<String> getList(Map<?, ?> map, String key) {
        Object o = map.get(key);
        if (o instanceof List) {
            return (List<String>) o;
        } else if (o instanceof String string) {
            return string.isEmpty() ? List.of() : List.of(string);
        } else {
            return List.of();
        }
    }

    private static List<String> getList(Map<String, Object> map,
                                        String key,
                                        String subkey) {
        return map.get(key) instanceof Map<?, ?> m ? getList(m, subkey) : List.of();
    }

    @SuppressWarnings("unchecked")
    private static Map<String, String> getMap(Map<?, ?> map, String key) {
        Object o = map.get(key);
        if (o instanceof Map<?, ?> m) {
            return (Map<String, String>) m;
        } else {
            return Map.of();
        }
    }

    private static String getString(Map<String, Object> map, String key) {
        Object o = map.get(key);
        if (o instanceof String string) {
            return string;
        } else {
            return "";
        }
    }

    private static Boolean getBoolean(Map<String, Object> map, String key) {
        Object o = map.get(key);
        if (o instanceof Boolean b) {
            return b;
        } else {
            return null;
        }
    }

    private static final Map<String, String> STATES = Map.of(
            "OK", "OK",
            "EMPTY_RESPONSE", "keine Treffer",
            "ERROR", "Fehler",
            "TIMEOUT", "Zeitüberschreitung"
    );

    private static final Map<String, String> MATERIALS = new HashMap<>() {
        {
            put("", "nicht belegt");
            put("text", "gedruckte Ressource");
            put("online_resource", "Online-Ressource");
            put("electronic_resource", "elektronische Ressource");
            put("microform", "Mikroform");
            put("projected_graphic", "Dia");
            put("non_projected_graphic", "Bildvorlage");
            put("motion_picture", "bewegte Bilder");
            put("tactile", "Blindenschrift");
            put("notated_music", "Notenschrift");
            put("remote_sensing_image", "Satellitenbilder");
            put("sound_recording", "Tonaufnahme");
            put("video_recording", "Bildaufnahme");
            put("kit", "Bausatz");
            put("map", "Landkarte");
            put("globe", "Globus");
            put("unspecified", "nicht spezifiziert");
        }
    };
}
