package org.xbib.interlibrary.http.server.search;

import java.io.IOException;
import java.util.List;
import org.xbib.datastructures.common.MultiMap;
import org.xbib.interlibrary.action.sru.DefaultSearchRequester;
import org.xbib.interlibrary.action.sru.Parameter;
import org.xbib.interlibrary.action.sru.SearchRequester;
import org.xbib.interlibrary.action.sru.SearchRetrieveAction;
import org.xbib.interlibrary.action.sru.SearchRetrieveRequest;
import org.xbib.interlibrary.action.sru.SearchRetrieveResponse;
import org.xbib.interlibrary.api.ApplicationModule;
import org.xbib.interlibrary.common.Interlibrary;
import org.xbib.interlibrary.common.InterlibraryModule;

public class SearchRetrieveModule implements InterlibraryModule {

    private Interlibrary interlibrary;

    public SearchRetrieveModule() {
    }

    @Override
    public String getName() {
        return "SearchRetrieveModule";
    }

    @Override
    public List<Class<? extends ApplicationModule>> dependsOn() {
        return null;
    }

    @Override
    public void setInterlibrary(Interlibrary interlibrary) {
        this.interlibrary = interlibrary;
    }

    @Override
    public Interlibrary getInterlibrary() {
        return interlibrary;
    }

    public SearchRetrieveResponse searchRetrieve(MultiMap<String, Object> params) {
        SearchRetrieveRequest.Builder builder = SearchRetrieveRequest.builder();
        if (params.containsKey(SearchRetrieveParameters.SERVICE)) {
            for (Object service : params.get(SearchRetrieveParameters.SERVICE)) {
                if (!"all".equals(service)) {
                    builder.setServices(service.toString());
                }
            }
        }
        // optional: requester
        SearchRequester requester = new DefaultSearchRequester(params.getString(SearchRetrieveParameters.REQUESTER, ""));
        builder.setRequester(requester);
        builder.setSearchString(params.getString(SearchRetrieveParameters.QUERY));
        builder.setOffset(params.getInteger(SearchRetrieveParameters.OFFSET, 0));
        builder.setSize(params.getInteger(SearchRetrieveParameters.SIZE, 10));
        // SRU parameters
        builder.setCQLQuery(params.getString(Parameter.QUERY));
        builder.setExtraRequestData(params.getString(Parameter.EXTRA_REQUEST_DATA));
        builder.setFilter(params.getString(Parameter.FILTER));
        builder.setStartRecord(params.getInteger(Parameter.START_RECORD, 1));
        builder.setMaximumRecords(params.getInteger(Parameter.MAXIMUM_RECORDS, 10));
        builder.setStyleSheet(stylesheet(params));
        builder.setDefaultStyleSheet(defaultStylesheet(params));
        builder.setFacetCount(params.getString(Parameter.FACET_COUNT));
        builder.setFacetStart(params.getString(Parameter.FACET_START));
        builder.setFacetLimit(params.getString(Parameter.FACET_LIMIT));
        builder.setFacetSort(params.getString(Parameter.FACET_SORT));
        builder.setEncoding(params.getString(Parameter.ENCODING));
        builder.setRecordPacking(params.getString(Parameter.RECORD_PACKING));
        builder.setRecordSchema(params.getString(Parameter.RECORD_SCHEMA));
        // our parameetrs
        builder.setEndpoint(params.getString(Parameter.ENDPOINT));
        return interlibrary.executeReduce(SearchRetrieveAction.INSTANCE, builder.build());
    }

    private String stylesheet(MultiMap<String,Object> params) {
        return params.getString(Parameter.RECORD_SCHEMA, "mods") + "/" +
                params.getString("index", "_all").replaceAll("\\d+$","") + ".xsl";
    }

    private String defaultStylesheet(MultiMap<String,Object> params) {
        return params.getString(Parameter.RECORD_SCHEMA, "mods") + "/" + "_all.xsl";
    }

    @Override
    public void run() {
    }

    @Override
    public void close() throws IOException {
    }
}
